package com.nttdata.wdp.green.carbonaware;

import java.io.IOException;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.net.URI;
import java.time.Duration;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;


public class CarbonAwareSDK{

  private static final URI ENDPOINT = URI.create("http://localhost:8080/emissions/forecasts/current?location=JP-TK");

  private final HttpClient client;

  private final HttpRequest request;

  private final ObjectMapper objectMapper;

  public CarbonAwareSDK(Duration connectTimeout, Duration requestTimeout){
    client = HttpClient.newBuilder()
                       .connectTimeout(connectTimeout)
                       .build();
    request = HttpRequest.newBuilder(ENDPOINT)
                         .GET()
                         .timeout(requestTimeout)
                         .build();
    objectMapper = new ObjectMapper();
  }

  public double carbonIntensity() throws IOException, InterruptedException{
    String result = client.send(request, HttpResponse.BodyHandlers.ofString())
                          .body();
    JsonNode node = objectMapper.readTree(result);
    return node.get(0)
               .get("optimalDataPoint")
               .get("value")
               .asDouble();
  }

}
