package com.nttdata.wdp.green.carbonaware;

import java.lang.instrument.Instrumentation;
import java.io.IOException;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.StringJoiner;
import java.util.Timer;
import java.util.TimerTask;


public class Main extends TimerTask{

  private static final long DELAY = 3_000;

  private static final long PERIOD = 60_000;

  private static final Duration connectTimeout = Duration.ofSeconds(5);

  private static final Duration requestTimeout = Duration.ofSeconds(10);

  private final Redfish redfish;

  private final CarbonAwareSDK sdk;

  private double gCO2eq_kWh;

  public Main(boolean isDaemon){
    redfish = new Redfish(connectTimeout, requestTimeout);
    sdk = new CarbonAwareSDK(connectTimeout, requestTimeout);
    gCO2eq_kWh = 0.0d;

    var timer = new Timer("Carbon Aware task", isDaemon);
    timer.scheduleAtFixedRate(this, DELAY, PERIOD);

    Thread report = new Thread(() -> System.out.println("Total: " + Double.toString(gCO2eq_kWh)));
    Runtime.getRuntime().addShutdownHook(report);
  }

  @Override
  public void run(){
    try{
      String label = LocalDateTime.now().format(DateTimeFormatter.ISO_LOCAL_DATE_TIME);
      double powerConsumptionInWatts = redfish.powerConsumedWatts();
      double carbonIntensity = sdk.carbonIntensity();
      double carbonEmission = powerConsumptionInWatts  / 1000.0d * carbonIntensity * PERIOD / 3_600_000.0d;
      StringJoiner joiner = new StringJoiner(",");
      joiner.add(label)
            .add(Double.toString(powerConsumptionInWatts))
            .add(Double.toString(carbonIntensity))
            .add(Double.toString(carbonEmission));
      System.out.println(joiner.toString());
      gCO2eq_kWh += carbonEmission;
    }
    catch(IOException | InterruptedException e){
      e.printStackTrace();
    }
  }

  public static void main(String[] args){
    new Main(false);
  }

  public static void agentmain(String agentArgs, Instrumentation inst){
    premain(agentArgs, inst);
  }

  public static void premain(String agentArgs, Instrumentation inst){
    new Main(true);
  }

}
