package com.nttdata.wdp.green.carbonaware;

import java.io.IOException;
import java.net.Authenticator;
import java.net.PasswordAuthentication;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.net.URI;
import java.time.Duration;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;


public class Redfish{

  private static final URI ENDPOINT = URI.create("https://idrac/redfish/v1/Chassis/System.Embedded.1/Power/PowerControl");

  private static final String IDRAC_USER = "username";

  private static final String IDRAC_PASSWORD = "password";

  private final HttpClient client;

  private final HttpRequest request;

  private final ObjectMapper objectMapper;

  public Redfish(Duration connectTimeout, Duration requestTimeout){
    Authenticator authenticator = new Authenticator(){
      @Override
      protected PasswordAuthentication getPasswordAuthentication() {
        return new PasswordAuthentication(IDRAC_USER, IDRAC_PASSWORD.toCharArray());
      }
    };

    client = HttpClient.newBuilder()
                       .connectTimeout(connectTimeout)
                       .authenticator(authenticator)
                       .build();
    request = HttpRequest.newBuilder(ENDPOINT)
                         .GET()
                         .timeout(requestTimeout)
                         .build();
    objectMapper = new ObjectMapper();
  }

  public double powerConsumedWatts() throws IOException, InterruptedException{
    String result = client.send(request, HttpResponse.BodyHandlers.ofString())
                          .body();
    JsonNode node = objectMapper.readTree(result);
    return node.get("PowerConsumedWatts").asDouble();
  }

}
